@@ -7,37 +7,30 @@ from diffprivlib.mechanisms.transforms import RoundedInteger
 
 class TestRoundedInteger(TestCase):
     def test_not_none(self):
-        mech = RoundedInteger(Laplace())
+        mech = RoundedInteger(Laplace(epsilon=1, sensitivity=1))
         self.assertIsNotNone(mech)
         _mech = mech.copy()
         self.assertIsNotNone(_mech)
 
     def test_class(self):
         from diffprivlib.mechanisms import DPMachine
+        from diffprivlib.mechanisms.transforms import DPTransformer
+
         self.assertTrue(issubclass(RoundedInteger, DPMachine))
+        self.assertTrue(issubclass(RoundedInteger, DPTransformer))
 
     def test_no_parent(self):
         with self.assertRaises(TypeError):
             RoundedInteger()
 
-    def test_empty_mechanism(self):
-        mech = RoundedInteger(Laplace())
-        with self.assertRaises(ValueError):
-            mech.randomise(1)
-
-    def test_set_epsilon_locally(self):
-        mech = RoundedInteger(Laplace().set_sensitivity(1))
-        mech.set_epsilon(1)
-        self.assertIsNotNone(mech)
-
     def test_randomise(self):
-        mech = RoundedInteger(Laplace().set_sensitivity(1).set_epsilon(1))
+        mech = RoundedInteger(Laplace(epsilon=1, sensitivity=1))
         self.assertIsInstance(mech.randomise(1), int)
 
     def test_distrib(self):
         epsilon = np.log(2)
         runs = 10000
-        mech = RoundedInteger(Laplace().set_sensitivity(1).set_epsilon(1))
+        mech = RoundedInteger(Laplace(epsilon=epsilon, sensitivity=1))
         count = [0, 0]
 
         for _ in range(runs):
